/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"


// Shows how to create named versions when committing transactions and how to
// iterate through a list of versions.  For additional information, see 
// the "Named Versions" section of Getting Started with Matisse.

void listVersions(MtDatabase* db, NSString *msg)
{
  int i;
  NSString *vername;
  MtVersionEnumerator* viter = [db versionEnumerator];

  printf("%s\n%s has the following versions:\n",
	 [msg UTF8String],
	 [[db description] UTF8String]);
  i = 0;
  while ((vername = [viter nextObject]) != nil) {
    printf("%d: %s\n", (i+1), [vername UTF8String]);
    i++;
  }
  printf("%d named version(s)\n", i);

  // close when done.
  [viter close];
}

int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname;
  NSString *versionname;

  if (argc < 3) {
    printf("Usage: %s <host> <database>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  @try {
    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname];

    [db open];

    [db startTransaction];

    listVersions(db, @"Versions before regular commit:");

    [db commit];

    [db startTransaction:MT_MIN_TRAN_PRIORITY];

    listVersions(db, @"Versions after regular commit:");

    versionname = [db commit:@"today_"];

    printf("Commit and preserve version named %s\n",
	   [versionname UTF8String]);

    [db startVersionAccess:versionname];

    listVersions(db, @"Versions after named commit:");

    printf("Version access to %s\n",
	   [versionname UTF8String]);

    [db endVersionAccess];

    [db startTransaction];
    printf("Delete version named %s\n",
	   [versionname UTF8String]);
    [db removeVersionAccess:versionname];


    listVersions(db, @"Versions after remove version:");

    [db commit];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
