/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

#import "Person.h"
#import "Employee.h"

// A simple example which shows how to use generated code for user classes.
int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname;
  
  if (argc < 3) {
    printf("Usage: %s <host> <database>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  @try {

    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the "examples.objc_examples.chaps_4_5"
    // namespace.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init: @"" : @"examples.objc_examples.chaps_4_5"];
    db = [(MtDatabase *)[MtDatabase alloc] init : hostname : dbname andFactory: factory];

    [db open];

    [db startTransaction];

    printf("\nCreating objects...\n");
    // create a new Person
    Person* p = [[Person alloc] init: db];
    // modify attributes
    [p setFirstName : @"John"];
    [p setLastName : @"Smith"];
    [p setAge : 42];
    printf("%s object created\n", [[p description] UTF8String]);

    // create a new Employee
    Employee* e = [[Employee alloc] init: db];
    // set attributes
    [e setFirstName : @"Jane"];
    [e setLastName: @"Jones"];
    // age is nullable, so leave unset

    // create timestamp from a string representation
    NSDate *ts = [NSDate dateWithString : @"2012-01-06 00:00:00 +0000" ];
    [e setHireDate : ts];
    NSDecimalNumber* num = [NSDecimalNumber decimalNumberWithString : @"100000.00"];
    [e setSalary: num];
    printf("%s object created\n", [[e description] UTF8String]);

    [db commit];


    [db startVersionAccess];

    // list all Persons
    printf("\nEnumerating objects...\n");
    printf("%d Persons in the database %s\n", [Person instanceNumber : db],
	   [[db description] UTF8String]);

    // open an iterator on all the instances of Person and all
    // subclasses; if you wish to exclude subclasses, use
    // [Person ownInstanceEnumerator] instead
    Person* x;
    MtObjectEnumerator* piter = [Person instanceEnumerator:db];
    while ((x = (Person*)[piter nextObject]) != nil) {
      // show attributes and name of class
      printf("%s %s is a %s\n", 
	     [[x firstName] UTF8String],
	     [[x lastName] UTF8String],
	     [[[x getMtClass ] mtFullName] UTF8String]);
    }
    [piter close];
    [db endVersionAccess];

    [db startTransaction:MT_MAX_TRAN_PRIORITY];
    printf("\nRemoving created objects...\n");
    //[p remove];
    [p deepRemove];
    //[e remove];
    [e deepRemove];

    printf("%d Person(s) in the database %s\n", [Person instanceNumber:db],
	   [[db description] UTF8String]);

    [db commit];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
